@@ -1,4 +1,4 @@
-# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
+# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4; encoding:utf-8 -*-
 #
 # Copyright 2002 Ben Escoto <ben@emerose.org>
 # Copyright 2007 Kenneth Loafman <kenneth@loafman.com>
@@ -21,15 +21,12 @@
 
 u"""Classes and functions on collections of backup volumes"""
 
-from past.builtins import cmp
-from builtins import filter
 from builtins import str
 from builtins import zip
 from builtins import map
 from builtins import range
 from builtins import object
 
-import types
 import sys
 
 from duplicity import log
@@ -37,7 +34,7 @@ from duplicity import file_naming
 from duplicity import path
 from duplicity import util
 from duplicity import dup_time
-from duplicity import globals
+from duplicity import config
 from duplicity import manifest
 from duplicity import util
 from duplicity.gpg import GPGError
@@ -118,7 +115,13 @@ class BackupSet(object):
         else:
             assert pr.volume_number is not None
             assert pr.volume_number not in self.volume_name_dict, \
-                (self.volume_name_dict, filename)
+                u"""Volume %d is already in the volume list as "%s".
+                "%s" has the same volume number.
+                Please check your command line and retry.""" % (
+                    pr.volume_number,
+                    util.fsdecode(self.volume_name_dict[pr.volume_number]),
+                    util.fsdecode(filename)
+                )
             self.volume_name_dict[pr.volume_number] = filename
 
         return True
@@ -148,12 +151,15 @@ class BackupSet(object):
         u"""
         Add local and remote manifest filenames to backup set
         """
-        assert not self.remote_manifest_name, (self.remote_manifest_name,
-                                               remote_filename)
+        assert not self.remote_manifest_name, \
+            u"Cannot set filename of remote manifest to %s; already set to %s." % (
+                remote_filename,
+                self.remote_manifest_name,
+            )
         self.remote_manifest_name = remote_filename
 
-        if self.action not in [u"collection-status", u"replicate"]:
-            local_filename_list = globals.archive_dir_path.listdir()
+        if self.action != u"replicate":
+            local_filename_list = config.archive_dir_path.listdir()
         else:
             local_filename_list = []
         for local_filename in local_filename_list:
@@ -163,7 +169,7 @@ class BackupSet(object):
                     pr.start_time == self.start_time and
                     pr.end_time == self.end_time):
                 self.local_manifest_path = \
-                    globals.archive_dir_path.append(local_filename)
+                    config.archive_dir_path.append(local_filename)
 
                 self.set_files_changed()
                 break
@@ -179,8 +185,8 @@ class BackupSet(object):
         except Exception:
             log.Debug(_(u"BackupSet.delete: missing %s") % [util.fsdecode(f) for f in rfn])
             pass
-        if self.action not in [u"collection-status", u"replicate"]:
-            local_filename_list = globals.archive_dir_path.listdir()
+        if self.action != u"replicate":
+            local_filename_list = config.archive_dir_path.listdir()
         else:
             local_filename_list = []
         for lfn in local_filename_list:
@@ -189,7 +195,7 @@ class BackupSet(object):
                     pr.start_time == self.start_time and
                     pr.end_time == self.end_time):
                 try:
-                    globals.archive_dir_path.append(lfn).delete()
+                    config.archive_dir_path.append(lfn).delete()
                 except Exception:
                     log.Debug(_(u"BackupSet.delete: missing %s") % [util.fsdecode(f) for f in lfn])
                     pass
@@ -415,16 +421,16 @@ class BackupChain(object):
         l = []
         for s in self.get_all_sets():
             if s.time:
-                type = u"full"
+                btype = u"full"
                 time = s.time
             else:
-                type = u"inc"
+                btype = u"inc"
                 time = s.end_time
             if s.encrypted:
                 enc = u"enc"
             else:
                 enc = u"noenc"
-            l.append(u"%s%s %s %d %s" % (prefix, type, dup_time.timetostring(time), (len(s)), enc))
+            l.append(u"%s%s %s %d %s" % (prefix, btype, dup_time.timetostring(time), (len(s)), enc))
         return l
 
     def __str__(self):
@@ -443,12 +449,12 @@ class BackupChain(object):
 
         for s in self.get_all_sets():
             if s.time:
-                type = _(u"Full")
+                btype = _(u"Full")
                 time = s.time
             else:
-                type = _(u"Incremental")
+                btype = _(u"Incremental")
                 time = s.end_time
-            l.append(set_schema % (type, dup_time.timetopretty(time), len(s)))
+            l.append(set_schema % (btype, dup_time.timetopretty(time), len(s)))
 
         l.append(u"-------------------------")
         return u"\n".join(l)
@@ -484,14 +490,14 @@ class SignatureChain(object):
         Return new SignatureChain.
 
         local should be true iff the signature chain resides in
-        globals.archive_dir_path and false if the chain is in
-        globals.backend.
+        config.archive_dir_path and false if the chain is in
+        config.backend.
 
-        @param local: True if sig chain in globals.archive_dir_path
+        @param local: True if sig chain in config.archive_dir_path
         @type local: Boolean
 
         @param location: Where the sig chain is located
-        @type location: globals.archive_dir_path or globals.backend
+        @type location: config.archive_dir_path or config.backend
         """
         if local:
             self.archive_dir_path, self.backend = location, None
@@ -669,10 +675,7 @@ class CollectionsStatus(object):
              (self.backend.__class__.__name__,),
              _(u"Archive dir: %s") % (self.archive_dir_path.uc_name if self.archive_dir_path else u'None',)]
 
-        l.append(u"\n" +
-                 ngettext(u"Found %d secondary backup chain.",
-                          u"Found %d secondary backup chains.",
-                          len(self.other_backup_chains))
+        l.append(u"\n" + _(u"Found %d secondary backup chain(s).")
                  % len(self.other_backup_chains))
         for i in range(len(self.other_backup_chains)):
             l.append(_(u"Secondary chain %d of %d:") %
@@ -688,14 +691,10 @@ class CollectionsStatus(object):
             l.append(_(u"No backup chains with active signatures found"))
 
         if self.orphaned_backup_sets or self.incomplete_backup_sets:
-            l.append(ngettext(u"Also found %d backup set not part of any chain,",
-                              u"Also found %d backup sets not part of any chain,",
-                              len(self.orphaned_backup_sets))
-                     % (len(self.orphaned_backup_sets),))
-            l.append(ngettext(u"and %d incomplete backup set.",
-                              u"and %d incomplete backup sets.",
-                              len(self.incomplete_backup_sets))
-                     % (len(self.incomplete_backup_sets),))
+            l.append(_(u"Also found %d backup set(s) not part of any chain,")
+                     % len(self.orphaned_backup_sets))
+            l.append(_(u"and %d incomplete backup set(s).")
+                     % len(self.incomplete_backup_sets))
             # TRANSL: "cleanup" is a hard-coded command, so do not translate it
             l.append(_(u'These may be deleted by running duplicity with the '
                        u'"cleanup" command.'))
@@ -716,20 +715,16 @@ class CollectionsStatus(object):
 
         # get remote filename list
         backend_filename_list = self.backend.list()
-        log.Debug(ngettext(u"%d file exists on backend",
-                           u"%d files exist on backend",
-                           len(backend_filename_list)) %
-                  len(backend_filename_list))
+        log.Debug(_(u"%d file(s) exists on backend")
+                  % len(backend_filename_list))
 
         # get local filename list
-        if self.action not in [u"collection-status", u"replicate"]:
+        if self.action != u"replicate":
             local_filename_list = self.archive_dir_path.listdir()
         else:
             local_filename_list = []
-        log.Debug(ngettext(u"%d file exists in cache",
-                           u"%d files exist in cache",
-                           len(local_filename_list)) %
-                  len(local_filename_list))
+        log.Debug(_(u"%d file(s) exists in cache")
+                  % len(local_filename_list))
 
         # check for partial backups
         partials = []
@@ -802,20 +797,12 @@ class CollectionsStatus(object):
         assert self.values_set
 
         if self.local_orphaned_sig_names:
-            log.Warn(ngettext(u"Warning, found the following local orphaned "
-                              u"signature file:",
-                              u"Warning, found the following local orphaned "
-                              u"signature files:",
-                              len(self.local_orphaned_sig_names)) + u"\n" +
+            log.Warn(_(u"Warning, found the following local orphaned signature file(s):") + u"\n" +
                      u"\n".join(map(util.fsdecode, self.local_orphaned_sig_names)),
                      log.WarningCode.orphaned_sig)
 
         if self.remote_orphaned_sig_names:
-            log.Warn(ngettext(u"Warning, found the following remote orphaned "
-                              u"signature file:",
-                              u"Warning, found the following remote orphaned "
-                              u"signature files:",
-                              len(self.remote_orphaned_sig_names)) + u"\n" +
+            log.Warn(_(u"Warning, found the following remote orphaned signature file(s):") + u"\n" +
                      u"\n".join(map(util.fsdecode, self.remote_orphaned_sig_names)),
                      log.WarningCode.orphaned_sig)
 
@@ -828,11 +815,7 @@ class CollectionsStatus(object):
                        u"from aborted session"), log.WarningCode.incomplete_backup)
 
         if self.orphaned_backup_sets:
-            log.Warn(ngettext(u"Warning, found the following orphaned "
-                              u"backup file:",
-                              u"Warning, found the following orphaned "
-                              u"backup files:",
-                              len(self.orphaned_backup_sets)) + u"\n" +
+            log.Warn(_(u"Warning, found the following orphaned backup file(s):") + u"\n" +
                      u"\n".join(map(str, self.orphaned_backup_sets)),
                      log.WarningCode.orphaned_backup)
 
@@ -855,7 +838,7 @@ class CollectionsStatus(object):
             Try adding filename to existing sets, or make new one
             """
             pr = file_naming.parse(filename)
-            for set in sets:
+            for set in sets:  # pylint: disable=redefined-builtin
                 if set.add_filename(filename, pr):
                     log.Debug(_(u"File %s is part of known set") % (util.fsdecode(filename),))
                     break
@@ -873,7 +856,7 @@ class CollectionsStatus(object):
 
         chains, orphaned_sets = [], []
 
-        def add_to_chains(set):
+        def add_to_chains(set):  # pylint: disable=redefined-builtin
             u"""
             Try adding set to existing chains, or make new one
             """
@@ -894,22 +877,22 @@ class CollectionsStatus(object):
                     orphaned_sets.append(set)
         for s in sets:
             add_to_chains(s)
-        return (chains, orphaned_sets, incomplete_sets)
+        return chains, orphaned_sets, incomplete_sets
 
     def get_sorted_sets(self, set_list):
         u"""
         Sort set list by end time, return (sorted list, incomplete)
         """
         time_set_pairs, incomplete_sets = [], []
-        for set in set_list:
+        for set in set_list:  # pylint: disable=redefined-builtin
             if not set.is_complete():
                 incomplete_sets.append(set)
             elif set.type == u"full":
                 time_set_pairs.append((set.time, set))
             else:
                 time_set_pairs.append((set.end_time, set))
-        time_set_pairs.sort()
-        return ([p[1] for p in time_set_pairs], incomplete_sets)
+        time_set_pairs.sort(key=lambda x: x[0])
+        return [p[1] for p in time_set_pairs], incomplete_sets
 
     def get_signature_chains(self, local, filelist=None):
         u"""
@@ -923,7 +906,7 @@ class CollectionsStatus(object):
             if filelist is not None:
                 return filelist
             elif local:
-                if self.action not in [u"collection-status", u"replicate"]:
+                if self.action != u"replicate":
                     return self.archive_dir_path.listdir()
                 else:
                     return []
@@ -960,7 +943,7 @@ class CollectionsStatus(object):
                     break
             else:
                 orphaned_filenames.append(sig_filename)
-        return (chains, orphaned_filenames)
+        return chains, orphaned_filenames
 
     def get_sorted_chains(self, chain_list):
         u"""
@@ -1047,7 +1030,7 @@ class CollectionsStatus(object):
                          dup_time.timetostring(oldest.start_time))
             return oldest
 
-    def get_extraneous(self, extra_clean):
+    def get_extraneous(self):
         u"""
         Return list of the names of extraneous duplicity files
 
@@ -1059,15 +1042,6 @@ class CollectionsStatus(object):
         local_filenames = []
         remote_filenames = []
         ext_containers = self.orphaned_backup_sets + self.incomplete_backup_sets
-        if extra_clean:
-            old_sig_chains = self.all_sig_chains[:]
-            if self.matched_chain_pair:
-                matched_sig_chain = self.matched_chain_pair[0]
-                for sig_chain in self.all_sig_chains:
-                    if (sig_chain.start_time == matched_sig_chain.start_time and
-                            sig_chain.end_time == matched_sig_chain.end_time):
-                        old_sig_chains.remove(sig_chain)
-            ext_containers += old_sig_chains
         for set_or_chain in ext_containers:
             if set_or_chain.backend:
                 remote_filenames.extend(set_or_chain.get_filenames())
@@ -1163,7 +1137,7 @@ class CollectionsStatus(object):
         if len(self.all_backup_chains) < n:
             return None
 
-        sorted = self.all_backup_chains[:]
+        sorted = self.all_backup_chains[:]  # pylint: disable=redefined-builtin
         sorted.sort(key=lambda x: x.get_first().time)
 
         sorted.reverse()
@@ -1216,6 +1190,7 @@ class CollectionsStatus(object):
         specified_file_backup_set = []
         specified_file_backup_type = []
 
+        modified_filepath = util.fsencode(modified_filepath)
         for bs in all_backup_set:
             filelist = [fileinfo[1] for fileinfo in bs.get_files_changed()]
             if modified_filepath in filelist:
@@ -1225,6 +1200,16 @@ class CollectionsStatus(object):
 
         return FileChangedStatus(filepath, list(zip(specified_file_backup_type, specified_file_backup_set)))
 
+    def get_all_file_changed_records(self, set_index):
+        u"""
+        Returns file changes in the specific backup set
+        """
+        if not self.matched_chain_pair:
+            return u""
+
+        all_backup_set = list(reversed(self.matched_chain_pair[1].get_all_sets()))
+        return BackupSetChangesStatus(all_backup_set[set_index])
+
 
 class FileChangedStatus(object):
     def __init__(self, filepath, fileinfo_list):
@@ -1234,7 +1219,7 @@ class FileChangedStatus(object):
     def __str__(self):
         set_schema = u"%20s   %30s  %20s"
         l = [u"-------------------------",
-             _(u"File: %s") % (self.filepath),
+             _(u"File: %s") % self.filepath,
              _(u"Total number of backup: %d") % len(self.fileinfo_list),
              set_schema % (_(u"Type of backup set:"), _(u"Time:"), _(u"Type of file change:"))]
 
@@ -1242,10 +1227,28 @@ class FileChangedStatus(object):
             backup_type = s[0]
             backup_set = s[1]
             if backup_set.time:
-                type = _(u"Full")
+                type = _(u"Full")  # pylint: disable=redefined-builtin
             else:
                 type = _(u"Incremental")
             l.append(set_schema % (type, dup_time.timetopretty(backup_set.get_time()), backup_type.title()))
 
         l.append(u"-------------------------")
         return u"\n".join(l)
+
+
+class BackupSetChangesStatus(object):
+    def __init__(self, backup_set):
+        self.backup_set = backup_set
+
+    def __str__(self):
+        changed_files = self.backup_set.get_files_changed()
+        max_file_path_len = max([len(c[1]) for c in changed_files] + [5])
+        set_schema = u"%%-%ds  %%20s" % max_file_path_len
+        l = [u"-------------------------",
+             _(u" Backup set time: %s") % (self.backup_set.get_timestr()),
+             _(u"Total number of changes: %d") % len(changed_files),
+             set_schema % (_(u"File:"), _(u"Type of file change:"))] + \
+            [set_schema % (c[1].decode(u'utf-8'), c[0].decode(u'utf-8'))
+             for c in changed_files] + \
+            [u"-------------------------"]
+        return u"\n".join(l)
